/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;

/**

*/
public class InteractiveHorizonDemo_R2 implements KeyListener, ComponentListener
{
   private FrameBufferFrame fbf; // The event handlers need
   private Scene scene;          // access to these three fields.
   private int n = 9;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
   */
   public InteractiveHorizonDemo_R2()
   {
      // Define initial dimensions for a FrameBuffer.
      final int fbWidth  = 512;
      final int fbHeight = 512;

      // Create a FrameBufferFrame holding a FrameBufferPanel.
      fbf = new FrameBufferFrame("Renderer 2", fbWidth, fbHeight);

      // Make this InteractiveCube_R2 object the event
      // handler for events from the FrameBufferFrame.
      fbf.addKeyListener(this);
      fbf.addComponentListener(this);

      Pipeline.doClipping = true;

      // Create the Scene object that we shall render.
      scene = new Scene();

      // Create the Model object.
      Model model = new PrivateModel(n);

      // Add the Model to the Scene.
      scene.addModel(model);
   }


   // Implement the KeyListener interface.
   @Override public void keyPressed(KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         Pipeline.debug = ! Pipeline.debug;
       //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
      }
      else if ('c' == c)
      {
         Pipeline.doClipping = ! Pipeline.doClipping;
         System.out.print("Clipping is turned ");
         System.out.println(Pipeline.doClipping ? "On" : "Off");
      }
      else if ('p' == c)
      {
         scene.camera.perspective = ! scene.camera.perspective;
         String p = scene.camera.perspective ? "perspective" : "orthographic";
         System.out.println("Using " + p + " projection");
      }
      else if ('x' == c)
      {
         for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
         {
            scene.modelList.get(0).vertexList.get(ls.index[1]).x -= 0.1;
         }
      }
      else if ('X' == c)
      {
         for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
         {
            scene.modelList.get(0).vertexList.get(ls.index[1]).x += 0.1;
         }
      }
      else if ('y' == c)
      {
         for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
         {
            scene.modelList.get(0).vertexList.get(ls.index[1]).y -= 0.1;
         }
      }
      else if ('Y' == c)
      {
         for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
         {
            scene.modelList.get(0).vertexList.get(ls.index[1]).y += 0.1;
         }
      }
      else if ('z' == c)
      {
         for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
         {
            scene.modelList.get(0).vertexList.get(ls.index[1]).z -= 1.0;
         }
      }
      else if ('Z' == c)
      {
         for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
         {
            scene.modelList.get(0).vertexList.get(ls.index[1]).z += 1.0;
         }
      }
      else if (('n' == c) || ('N' == c))
      {
         if ('n' == c)
            n--;
         else
            n++;
         scene.modelList.set(0,  new PrivateModel(n));
      }

      // Render again.
      FrameBuffer fb = fbf.fbp.getFrameBuffer();
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb.vp);
      fbf.fbp.update();
      fbf.repaint();
   }


   // Implement the ComponentListener interface.
   @Override public void componentMoved(ComponentEvent e){}
   @Override public void componentHidden(ComponentEvent e){}
   @Override public void componentShown(ComponentEvent e){}
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      // Get the new size of the FrameBufferPanel.
      int w = fbf.fbp.getWidth();
      int h = fbf.fbp.getHeight();

      // Create a new FrameBuffer that fits the new window size.
      FrameBuffer fb = new FrameBuffer(w, h);
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb.vp);
      fbf.fbp.setFrameBuffer(fb);
      fbf.fbp.update();
      fbf.repaint();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // an anonymous inner class constructor
         {
            public void run() // implement the Runnable interface
            {
               // Call the constructor that builds the gui.
               new InteractiveHorizonDemo_R2();
            }
         }
      );
   }


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, keys to move the horizon point along the x and y axes.");
      System.out.println("Use the z/Z, keys to change the length of the lines.");
      System.out.println("Use the n/N, keys to change the number of lines.");
      System.out.println("Use the 'c' key to toggle line clipping on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}//InteractiveHorizonDemo_R2


class PrivateModel extends Model
{
   PrivateModel(int n)
   {
      super();

      // Add geometry to the Model.
      for (int i = 0; i < n; i++)
      {
         for (int j = 0; j < n; j++)
         {
            addVertex(new Vertex(-1.0 + (2.0*i)/(n-1), -1.0 + (2.0*j)/(n-1), -1 ),
                      new Vertex(-1.0 + (2.0*i)/(n-1), -1.0 + (2.0*j)/(n-1), -10));
            addLineSegment(new LineSegment(2*((i * n) + j),
                                           2*((i * n) + j) + 1 ));
         }
      }
   }
}
