/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.io.File;
import java.awt.Color;
import java.awt.event.*;
import java.util.ArrayList;

/**

*/
@SuppressWarnings("serial")
public class InteractiveModels_R2 implements KeyListener, ComponentListener
{
   private FrameBufferFrame fbf; // The event handlers need
   private Scene scene;          // access to these fileds.
   private ArrayList<Model> modelArray = new ArrayList<>();
   private int currentModel = 1;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
   */
   public InteractiveModels_R2()
   {
      // Define initial dimensions for a FrameBuffer.
      final int fbWidth  = 1024;
      final int fbHeight = 1024;

      // Create a FrameBufferFrame holding a FrameBufferPanel.
      fbf = new FrameBufferFrame("Renderer 2", fbWidth, fbHeight);

      // Make this InteractiveModels_R2 object the event
      // handler for events from the FrameBufferFrame.
      fbf.addKeyListener(this);
      fbf.addComponentListener(this);


      // Create the Scene object that we shall render
      scene = new Scene();

      // Create several Model objects.
      modelArray.add( new ObjSimpleModel(new File("assets/apple.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/cow.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/galleon.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/teapot.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/cessna.obj")) );
      modelArray.add( new Sphere(1.0, 30, 30) );
      modelArray.add( new Cylinder(0.5, 1.0, 20, 20) );
      modelArray.add( new Torus(0.75, 0.25, 25, 25) );
      modelArray.add( new Cube2(15, 15, 15) );
      modelArray.add( new ObjSimpleModel(new File("assets/small_rhombicosidodecahedron.obj")) );
      modelArray.add( new PanelXY(-4, 4, -1, 3) );  // wall
      modelArray.add( new PanelXZ(-2, 2, -2, 0) );  // floor

      // Add three models to the Scene.
      int floor = modelArray.size() - 1;
      int wall  = modelArray.size() - 2;
      scene.addModel(modelArray.get(1),  // cow
                     modelArray.get(floor),
                     modelArray.get(wall));

      for (Vertex v : modelArray.get(wall).vertexList)
      {
         v.z -= 2;  // move back
      }
      for (Vertex v : modelArray.get(floor).vertexList)
      {
         v.y -= 1;  // move down
      }
      // Push the models away from where the camera is.
      for (Model m : modelArray)
      {
         for (Vertex v : m.vertexList)
         {
            v.z -= 2;
         }
      }
   }


   // Implement the KeyListener interface.
   @Override public void keyPressed(KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         modelArray.get(currentModel).debug = ! modelArray.get(currentModel).debug;
       //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
      }
      else if ( Character.isDigit(c) )
      {
         currentModel = Integer.parseInt( Character.toString(c) );
         scene.modelList.set(0, modelArray.get(currentModel));
      }
      else if ('c' == c)
      {
         Pipeline.doClipping = ! Pipeline.doClipping;
         System.out.print("Clipping is turned ");
         System.out.println(Pipeline.doClipping ? "On" : "Off");
      }
      else if ('p' == c)
      {
         scene.camera.perspective = ! scene.camera.perspective;
         String p = scene.camera.perspective ? "perspective" : "orthographic";
         System.out.println("Using " + p + " projection");
      }
      else if ('s' == c) // Scale the model 10% smaller.
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x /= 1.1;  // This does NOT work with
            v.y /= 1.1;  // perspective projection.
            v.z /= 1.1;
         }
      }
      else if ('S' == c) // Scale the model 10% larger.
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x *= 1.1;  // This does NOT work with
            v.y *= 1.1;  // perspective projection.
            v.z *= 1.1;
         }
      }
      else if ('x' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x -= 0.1;
         }
      }
      else if ('X' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x += 0.1;
         }
      }
      else if ('y' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.y -= 0.1;
         }
      }
      else if ('Y' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.y += 0.1;
         }
      }
      else if ('z' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.z -= 0.1;
         }
      }
      else if ('Z' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.z += 0.1;
         }
      }

      // Render again.
      FrameBuffer fb = fbf.fbp.getFrameBuffer();
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb.vp);
      fbf.fbp.update();
      fbf.repaint();
   }


   // Implement the ComponentListener interface.
   @Override public void componentMoved(ComponentEvent e){}
   @Override public void componentHidden(ComponentEvent e){}
   @Override public void componentShown(ComponentEvent e){}
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      // Get the new size of the FrameBufferPanel.
      int w = fbf.fbp.getWidth();
      int h = fbf.fbp.getHeight();

      // Create a new FrameBuffer that fits the new window size.
      FrameBuffer fb = new FrameBuffer(w, h);
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb.vp);
      fbf.fbp.setFrameBuffer(fb);
      fbf.fbp.update();
      fbf.repaint();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // an anonymous inner class constructor
         {
            public void run() // implement the Runnable interface
            {
               // Call the constructor that builds the gui.
               new InteractiveModels_R2();
            }
         }
      );
   }


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the digit keys '0' - '9' to choose a model.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to translate the model along the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the model.");
      System.out.println("Use the 'c' key to toggle line clipping on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
